import pygame, sys
from pygame.locals import *
from game import game 
from gameSprite import *
from gameScreen import gameScreen
from level import level
from splashScreen import splashScreen
from character import character
from enemy import *
from player import player

class PestControl(game):
    
    def __init__(self, main_screen):
        
        super(PestControl, self).__init__()
        
        self.main_screen = main_screen
        self.player = player(self.main_screen, 0, 300, 10, "sawblade.png")

        
    def init_game(self):
        
        self.player.reset()
    
    def play_level(self, level_num):
        
        self.level = level(level_num, self.main_screen)
        
        self.game_objects = []
        self.game_objects.append(self.player)
        self.game_objects.extend(self.level.enemies)
        self.game_objects.extend(self.level.obstacles)
        
        #Add barrier to prevent player/enemies from going past horizon
        self.game_objects.append(gameObject(self.main_screen,-1000,250,self.main_screen.window_width + 2000, 1, 0))
        
        self.moving_objects = []
        
        #Reset player position
        self.player.speed_x = 0
        self.player.speed_y = 0
        self.player.left = 0
        self.player.top = 300
        self.player.attacking = False
        
        #Always display this text for each level
        self.level.text_list.append("Level " + str(self.level.level_num) + "\n\nSave at least " + str(self.level.crystals_to_protect) + " Crystals!\n\n\n\n\n\nLives: " + str(self.player.lives))
        
        splashScreen(self.main_screen, self.level.text_list).show()
        
        self.level.start()
        self.state = "play"
        self.game_loop()
        
        for obj in self.game_objects:
            obj.cleanup()
        
        return self.state
    
    def update(self):
        
        #Level may need to send a new wave
        self.level.update(self.game_objects)
        
        #Get input from user
        self.player.update_controls()
        
        #Call each objects update() method, remove any objects if necesary
        to_remove = []
        for obj in self.game_objects:
            if(obj.do_remove):
                to_remove.append(obj)
            else:
                obj.update()    
        
        for obj_to_remove in to_remove:
            try:
                self.game_objects.remove(obj_to_remove)
                self.level.obstacles.remove(obj_to_remove)
            except:
                pass
            
        
        self.collision_detection()
        
      
        for obj in self.moving_objects:
            obj.move()
            
        #Determine if we've won or lost the level
        if(len(self.level.obstacles) < self.level.crystals_to_protect):
            self.state = "lose"
            self.player.lives -= 1
        
        if(self.level.time_level_ends <= time.time()):
            self.state = "win"
                
       
      
    
    def collision_detection(self):
        
        
        #Build list of moving objects
        self.moving_objects = []
    
        for obj in self.game_objects:
        
            if obj.movable:
                #if obj.speed_x != 0 or obj.speed_y != 0:
                self.moving_objects.append(obj)
        
                    
        #Look for collisions and resolve them if they occur
        for moving_object in self.moving_objects:
            
            for obj in self.game_objects:
                
                #print "Moving: " + str(moving_object.type) + " obj: " + str(obj.type)
                
                if obj == moving_object:
                    continue
                
                future_object = obj.get_future_rect();     
                if future_object.colliderect(moving_object.get_future_rect()): 
                    moving_object.did_collide(obj)
              
    
    def draw(self):
        
 
        #self.main_screen.screen.fill([0,0,255])
        self.level.draw_background(self.main_screen.screen)
        
        for obj in self.game_objects:
            obj.draw()
            
        self.draw_profiles()
        self.draw_status()
               
        pygame.display.flip()
        
    def draw_profiles(self):
        
        num_profiles = len(self.level.profiles)
        space_between_profiles = 10
        extra_space = space_between_profiles * (num_profiles - 1)
        profiles_width = (num_profiles * enemyProfile.profile_width) + extra_space
        
        draw_left_offset = int((self.main_screen.window_width / 2) - (profiles_width / 2))
        
        for profile in self.level.profiles:
            profile.draw(draw_left_offset, 60)
            draw_left_offset += enemyProfile.profile_width + space_between_profiles
            
    def draw_status(self):
        
        font = pygame.font.Font(None, 60)
        time_raw = int(self.level.time_level_ends - time.time())
        time_text = "Time: " + str(time_raw) + " Goal: " + str(self.level.crystals_to_protect)
        text = font.render(time_text, 1, (0, 0, 0))
        text_rect = text.get_rect()
        
        text_left = (self.main_screen.window_width / 2) - 160
        
        timer_rect = pygame.Rect(text_left, 5, text_rect.width, text_rect.height)
        
        self.main_screen.screen.blit(text, timer_rect)
        
        
        
    
    