import pygame, os, random, math
from pygame.locals import *
from character import *

class enemy(character):
    
    def __init__(self, gameWindow, speed, file_name, enemy_profile, target_list):
        
        super(enemy, self).__init__(gameWindow, 0, 0, speed, file_name)
        
        self.path_amount_moved = 0
        
        self.distance_traveled = 0
        self.distance_to_travel = 0
        self.movement_unit = 100
        
        self.type = "enemy"
        
        self.rage_value = 30
        self.weight = 30
        
        self.damage_speed = .2
        self.last_damage = 0
        
        #Nemesis mode modifies these attributes
        self.damage = 5
        self.damage_base = self.damage 
        self.speed_base = speed
        
        self.rage_multiplier_damage = 1
        self.rage_multiplier_speed = 2
        
        self.attack_target = 0
        
        #Death animations the same for all enemies
        death_animation = ["death1.png","death2.png","death3.png"]
        self.death_animation = animation(death_animation, .1)
        
        pygame.mixer.init()
        self.splat_channel = pygame.mixer.Channel(0)
        splat_file = os.path.join("data", "splat.wav")
        self.soundSplat = pygame.mixer.Sound(splat_file)
        
        self.enemy_profile = enemy_profile
        
        #self.get_random_start_pos()
        self.left = 0
        self.right = 0
        
        self.target_list = target_list
        self.target = None
        
    def get_random_start_pos(self):
        
        #Pick random starting position on the outside of the screen
        left = 0
        top = 0
        
        #Will either start on left side(0), bottom(1), or right side(2)
        side = math.floor(random.random() * 3)
        
        if(side == 0 ): #left side
            left = 0 - self.width
            top = 300 + math.floor(random.random() * (self.gameWindow.window_height - 300))
            
        elif(side == 1): #bottom
            top = self.gameWindow.window_height
            left = math.floor(random.random() * (self.gameWindow.window_width - self.width))
            
        elif(side == 2): #right side
            left = self.gameWindow.window_width
            top = 300 + math.floor(random.random() * (self.gameWindow.window_height - 300))
            
        return (left, top)
        
    def get_random_target(self):
        
        num_targets = len(self.target_list)
        
        if num_targets == 0:
            return 
            
        target_index = int(math.floor(random.random() * (num_targets)))
        potential_target = self.target_list[target_index]
        
        if potential_target.do_remove == True:
            return
        
        self.target = potential_target
        
    def set_pos(self, left, top):
        
        self.left = left
        self.top = top
        
        self.rect = pygame.Rect(left, top, self.width, self.height)
        
        
    def update(self):
        
        #Adjust enemy attribute if in Nemesis mode
        if self.enemy_profile.nemesis:
            self.speed = self.speed_base * self.rage_multiplier_speed
            self.damage = self.damage_base * self.rage_multiplier_damage
           
           
        if(self.attacking):
            
            self.inflict_damage()
            
        elif self.target is None or self.target.do_remove:
            
            self.get_random_target()
            
        elif(self.distance_traveled >= self.distance_to_travel):
            
            
            self.distance_traveled = 0
            self.path_set_direction()
            
    def did_collide(self, obj):
        
        if(obj.type == "enemy"):
            self.adjust_speed_for_collision(obj)
            return
        
        if(obj.type == "obstacle" and self.did_collide_hitbox(obj)):
            self.start_attack(obj)
        
        super(enemy, self).did_collide(obj)
        
    def start_attack(self, obj):
        
        self.attacking = True
        self.attack_target = obj
        
        self.speed_x = 0
        self.speed_y = 0
        self.distance_traveled = 0
        

    def inflict_damage(self):
        
        now = time.time()
        elapsed_time = now - self.last_damage
        
        if(elapsed_time >= self.damage_speed):
            
            if self.target.do_remove:
                self.attacking = False
            else: 
                self.attack_target.take_damage(self.damage)
                self.last_damage = now
                
                if(self.attack_target.hitpoints <= 0):
                    self.attacking = False           
        
    def move(self):
        super(enemy, self).move()
        
        self.distance_traveled += self.speed
        
    def kill(self):
        
        self.enemy_profile.add_rage(self.rage_value)
        
        self.dead = True
        
        #self.soundSplat.play()
        self.splat_channel.play(self.soundSplat)

         
        
    def path_set_direction(self):
        
        move_horizontal = False
        move_vertical = False
        distance_horizontal = self.target.left - self.left
        distance_vertical = self.target.top - self.top
        
        if(distance_horizontal == 0):
            move_vertical = True            
        elif(distance_vertical == 0):
            move_horizontal = True
        else:
            #randomly determine the direction
            random_val = (random.random() * 2) // 1
            if(random_val == 0):
                move_vertical = True
            else:
                move_horizontal = True
        
        if(move_vertical):
            self.speed_x = 0
            #Get the sign by dividing the distance by the absolute value of itself
            if(distance_vertical != 0):
                self.speed_y =  self.speed * (distance_vertical / abs(distance_vertical))
            
            self.distance_to_travel = min(abs(distance_vertical), self.movement_unit)
            
        elif(move_horizontal):
            self.speed_y = 0
            
            if(distance_horizontal != 0):
                self.speed_x = self.speed * (distance_horizontal / abs(distance_horizontal))
            
            self.distance_to_travel = min(abs(distance_horizontal), self.movement_unit)
            
class enemyProfile:
    
    profile_width = 150
    profile_height = 180
    
    def __init__(self, gameWindow, enemy_name, profile_image_happy, profile_image_rage):
    
        self.gameWindow = gameWindow    
        self.enemy_name = enemy_name
        #self.left = left
        #self.top = top
    
        self.enemy_rage = 0
        self.nemesis = False
        
        file_name_happy = os.path.join("data", profile_image_happy)
        self.image_happy = pygame.image.load(file_name_happy).convert()
        
        file_name_rage = os.path.join("data", profile_image_rage)
        self.image_rage = pygame.image.load(file_name_rage).convert()
        
        self.image = self.image_happy
        
        pygame.mixer.init()
        file_name = os.path.join("data", "69323__meowtek__hardbass1.wav")
        self.evil_sound = pygame.mixer.Sound(file_name)
        self.profile_channel = pygame.mixer.Channel(4)
 
    def add_rage(self, rage_amount):
       
        self.enemy_rage = min(self.enemy_rage + rage_amount, 100)
        
        
        
    def get_rage_string(self):
        
        if(self.enemy_rage >= 100):
            
            self.image = self.image_rage
            self.nemesis = True
            #self.profile_channel.play(self.evil_sound)
            return "NEMESIS"
        
        else:
            return "Rage: " + str(self.enemy_rage)
        
        
    def draw(self, left, top):
        
        #box with border
        margin = 5
        width = enemyProfile.profile_width
        height = enemyProfile.profile_height
        self.border_rect = pygame.Rect(left, top, width, height)
        self.main_rect = pygame.Rect(left+margin, top+margin, width - (margin*2), height - (margin*2))
        self.border_rect_color = (255,255,0)
        self.main_rect_color = (0,0,0)
        
        #image
        image_left = left + margin + 10
        image_top = top + margin + 10
        
        #text
        self.name_text_left = left+margin
        self.name_text_top = top + (height - margin - 50)
        self.name_text_width = width - (margin*2)
        self.name_text_height = 20
        
        #Rage text
        self.rage_text_left = left+margin
        self.rage_text_top = top + (height - margin - 30)
        self.rage_text_width = width - (margin*2)
        self.rage_text_height = 20
        
        pygame.draw.rect(self.gameWindow.screen, self.border_rect_color, self.border_rect)
        pygame.draw.rect(self.gameWindow.screen, self.main_rect_color, self.main_rect)
        
        #image
        self.gameWindow.screen.blit(self.image, (image_left, image_top))
        
        #Name text
        font = pygame.font.Font(None, 20)
        name_text = font.render(self.enemy_name, 1, (255, 255, 255))
        name_text_left = self.name_text_left + (self.name_text_width // 2) - (name_text.get_width() // 2)

        name_text_rect = pygame.Rect(name_text_left, self.name_text_top, name_text.get_width(), name_text.get_height())
        self.gameWindow.screen.blit(name_text, name_text_rect)
        
        #Rage text
        font = pygame.font.Font(None, 20)
        rage_string = self.get_rage_string()
        rage_text = font.render(rage_string, 1, (255, 255, 255))
        rage_text_left = self.rage_text_left + (self.rage_text_width // 2) - (rage_text.get_width() // 2)

        rage_text_rect = pygame.Rect(rage_text_left, self.rage_text_top, rage_text.get_width(), rage_text.get_height())
        self.gameWindow.screen.blit(rage_text, rage_text_rect)
        
class pig(enemy):
    
    def __init__(self, gameWindow, enemy_profile, target_list):
        
        speed = 1
        file_name = "pig_walk_down1.png"
        
        super(pig, self).__init__(gameWindow, speed, file_name, enemy_profile, target_list)
        
        self.rage_multiplier_damage = 3
        self.rage_multiplier_speed = 3
        
        self.hitbox_margin = 5
        
        moving_right_animation = ["pig_walk_right1.png","pig_walk_right2.png","pig_walk_right3.png","pig_walk_right4.png"]
        self.moving_right_animation = animation(moving_right_animation, .2)
        
        moving_left_animation = ["pig_walk_left1.png","pig_walk_left2.png","pig_walk_left3.png","pig_walk_left4.png"]
        self.moving_left_animation = animation(moving_left_animation, .2)
        
        moving_up_animation = ["pig_walk_up1.png","pig_walk_up2.png","pig_walk_up3.png","pig_walk_up4.png"]
        self.moving_up_animation = animation(moving_up_animation, .2)
        
        moving_down_animation = ["pig_walk_down1.png","pig_walk_down2.png","pig_walk_down3.png","pig_walk_down4.png"]
        moving_down_animation = animation(moving_down_animation, .2)
        
class bunny(enemy):
    
    def __init__(self, gameWindow, enemy_profile, target_list):
        
        speed = 1
        file_name = "bunny_walk_down1.png"
        
        super(bunny, self).__init__(gameWindow, speed, file_name, enemy_profile, target_list)
        
        self.rage_multiplier_damage = 3
        self.rage_multiplier_speed = 3
        
        #Bunnies need a smaller hitbox or they can't inflict damage
        self.hitbox_margin = 5
        
        moving_right_animation = ["bunny_walk_right1.png","bunny_walk_right2.png","bunny_walk_right3.png","bunny_walk_right4.png"]
        self.moving_right_animation = animation(moving_right_animation, .2)
        
        moving_left_animation = ["bunny_walk_left1.png","bunny_walk_left2.png","bunny_walk_left3.png","bunny_walk_left4.png"]
        self.moving_left_animation = animation(moving_left_animation, .2)
        
        moving_up_animation = ["bunny_walk_up1.png","bunny_walk_up2.png","bunny_walk_up3.png","bunny_walk_up4.png"]
        self.moving_up_animation = animation(moving_up_animation, .2)
        
        moving_down_animation = ["bunny_walk_down1.png","bunny_walk_down2.png","bunny_walk_down3.png","bunny_walk_down4.png"]
        moving_down_animation = animation(moving_down_animation, .2)
        
class llama(enemy):
    
    def __init__(self, gameWindow, enemy_profile, target_list):
        
        speed = 1
        file_name = "llama_walk_down1.png"
        
        super(llama, self).__init__(gameWindow, speed, file_name, enemy_profile, target_list)
        
        self.rage_multiplier_damage = 3
        self.rage_multiplier_speed = 3
        
        moving_right_animation = ["llama_walk_right1.png","llama_walk_right2.png","llama_walk_right3.png","llama_walk_right4.png"]
        self.moving_right_animation = animation(moving_right_animation, .2)
        
        moving_left_animation = ["llama_walk_left1.png","llama_walk_left2.png","llama_walk_left3.png","llama_walk_left4.png"]
        self.moving_left_animation = animation(moving_left_animation, .2)
        
        moving_up_animation = ["llama_walk_up1.png","llama_walk_up2.png","llama_walk_up3.png","llama_walk_up4.png"]
        self.moving_up_animation = animation(moving_up_animation, .2)
        
        moving_down_animation = ["llama_walk_down1.png","llama_walk_down2.png","llama_walk_down3.png","llama_walk_down4.png"]
        moving_down_animation = animation(moving_down_animation, .2)
        
class cow(enemy):
    
    def __init__(self, gameWindow, enemy_profile, target_list):
        
        speed = 1
        file_name = "cow_walk_down1.png"
        
        super(cow, self).__init__(gameWindow, speed, file_name, enemy_profile, target_list)
        
        self.rage_multiplier_damage = 3
        self.rage_multiplier_speed = 3
        
        moving_right_animation = ["cow_walk_right1.png","cow_walk_right2.png","cow_walk_right3.png","cow_walk_right4.png"]
        self.moving_right_animation = animation(moving_right_animation, .2)
        
        moving_left_animation = ["cow_walk_left1.png","cow_walk_left2.png","cow_walk_left3.png","cow_walk_left4.png"]
        self.moving_left_animation = animation(moving_left_animation, .2)
        
        moving_up_animation = ["cow_walk_up1.png","cow_walk_up2.png","cow_walk_up3.png","cow_walk_up4.png"]
        self.moving_up_animation = animation(moving_up_animation, .2)
        
        moving_down_animation = ["cow_walk_down1.png","cow_walk_down2.png","cow_walk_down3.png","cow_walk_down4.png"]
        moving_down_animation = animation(moving_down_animation, .2)
        
    
            
            
    
            