import pygame, os
from pygame.locals import * 
import math


class gameObject(pygame.sprite.Sprite):
    
    def __init__(self, gameWindow, left, top, width, height, speed):
        
        pygame.sprite.Sprite.__init__(self)
        
        self.left = left
        self.top = top
        self.width = width
        self.height = height
        
        self.type = "Object"
        self.weight = 1
        
        self.speed_x = 0
        self.speed_y = 0
        self.speed = speed
        
        self.did_collide_x = False
        self.did_collide_y = False
        self.collided_speed_x = 0
        self.collided_speed_y = 0
        
        self.rect = self.get_rect()
        
        self.gameWindow = gameWindow
        
        self.hitbox_margin = 10
        
        self.weight = 1000
        self.do_remove = False
        
        self.movable = False
        
    def move(self):
        
        if(self.did_collide_x):
            self.left += self.collided_speed_x
            self.did_collide_x = False
        else:
            self.left += self.speed_x
            
        if(self.did_collide_y):
            self.top += self.collided_speed_y
            self.did_collide_y = False
        else:
            self.top += self.speed_y
            
        self.rect = pygame.Rect(self.left, self.top, self.width, self.height)
                  
    
    def adjust_speed_for_collision(self, rect):
  
        #print "Collided! type: " + str(self.type) + " other: " + str(rect.type)
        #print "my left: " + str(self.left) + " my top: " + str(self.top) + " other left: " + str(rect.left) + " other top: " + str(rect.top)
        #print "my speed x: " +str(self.speed_x) + " my speed y: " + str(self.speed_y) + " other speed x: " + str(rect.speed_x) + " other speed y: " + str(rect.speed_y) 
        #X-Axis
        
        #Collide right side
        if self._will_collide_right(rect):
            #print "collided right"
            if(self.weight < rect.weight):
                self.collided_speed_x = rect.get_future_left() - self.get_right()
                self.did_collide_x = True
            else:
                rect.collided_speed_x = self.get_future_right() - rect.left
                rect.did_collide_x = True
                     
        #Collide left side
        elif self._will_collide_left(rect):
           # print "collided left"
            if(self.weight < rect.weight):
                self.collided_speed_x = rect.get_future_right() - self.left
                self.did_collide_x = True
            else:
                rect.collided_speed_x = self.get_future_left() - rect.get_right()
                rect.did_collide_x = True
            
        #Y-Axis
        
        #Collide bottom side
        if self._will_collide_bottom(rect):
          #  print "collided bottom"
            if(self.weight < rect.weight):
                self.collided_speed_y = rect.get_future_top() - self.get_bottom()
                self.did_collide_y = True
            else:
                rect.collided_speed_y = self.get_future_bottom() - rect.top
                rect.did_collide_y = True
            
        #Collide top side
        elif self._will_collide_top(rect):
        #    print "collided top"
            if(self.weight < rect.weight):
                self.collided_speed_y = rect.get_future_bottom() - self.top
                self.did_collide_y = True
            else:
                rect.collided_speed_y = self.get_future_top() - rect.get_bottom()
                rect.did_collide_y = True
                
        #if self.did_collide_x == False and self.did_collide_y == False and rect.did_collide_x == False and rect.did_collide_y == False:
        #    print "WHAAAAT??"
    
    def _will_collide_right(self, rect): 
        #print "future right: " + str(self.get_future_right()) + " >= rect.future left: " + str(rect.get_future_left()) + " and right: " + str(self.get_right()) + " <= rect.left: " + str(rect.left)
        return self.speed_x >= 0 and self.get_future_right() >= rect.get_future_left() and self.get_right() <= rect.left
    
    def _will_collide_left(self, rect):
        #print "future left: " + str(self.get_future_left()) + " <= rect.get_future_right: " + str(rect.get_future_right()) + " and left; " + str(self.left) + " >= rect.right: " + str(rect.get_right())
        return self.speed_x <= 0 and self.get_future_left() <= rect.get_future_right() and self.left >= rect.get_right()
    
    def _will_collide_top(self, rect):
        #print "future top: " + str(self.get_future_top()) + " <= rect.future_bottom: " + str(rect.get_future_bottom()) + " and self.top: " + str(self.top) + " >= rect.get_bottom: " + str(rect.get_bottom())
        return self.speed_y <= 0 and self.get_future_top() <= rect.get_future_bottom() and self.top >= rect.get_bottom()
    
    def _will_collide_bottom(self, rect):
        #print "my future bottom: " + str(self.get_future_bottom()) + " >= rect future top: " + str(rect.get_future_top()) + " and my bottom: " + str(self.get_bottom()) + " <= rect top: " + str(rect.top)
        return self.speed_y >= 0 and self.get_future_bottom() >= rect.get_future_top() and self.get_bottom() <= rect.top
    
    def get_right(self):
        return self.left + self.width
    
    def get_bottom(self):
        return self.top + self.height
    
    def get_future_left(self):
        return self.left + self.speed_x
    
    def get_future_top(self):
        return self.top + self.speed_y
    
    def get_future_right(self):
        return self.get_right() + self.speed_x
    
    def get_future_bottom(self):
        return self.get_bottom() + self.speed_y
    
    def get_rect(self):
        return pygame.Rect(self.left, self.top, self.width, self.height)
    
    def get_center_x(self):
        return self.left + (self.width / 2)
    
    def get_center_y(self):
        return self.top + (self.height / 2)
    
    def get_future_rect(self):
        return pygame.Rect(self.get_future_left(),self.get_future_top(),self.width,self.height)     
    
    def update(self):
        pass
    
    def get_hitbox(self):
        
        hitbox = gameObject(self.gameWindow, self.left+self.hitbox_margin, self.top+self.hitbox_margin, self.width-(self.hitbox_margin * 2), self.height-(self.hitbox_margin * 2), self.speed)
        hitbox.speed_x = self.speed_x
        hitbox.speed_y = self.speed_y
        hitbox.type = self.type
        hitbox.weight = self.weight
        
        return hitbox
        
    def did_collide(self, obj):
        
        hitbox = self.get_hitbox()
        obj_hitbox = obj.get_hitbox()
        
        future_hitbox = hitbox.get_future_rect()
        future_obj_hitbox= obj_hitbox.get_future_rect()
        
        #print str(self.type) + " collided with: " + str(obj.type)
        
        if(future_hitbox.colliderect(future_obj_hitbox)):
            #print "collision! Moving: " + str(self.type) + " obj: " + str(obj.type)
            
            hitbox.adjust_speed_for_collision(obj_hitbox)
            self.speed_x = hitbox.speed_x
            self.speed_y = hitbox.speed_y
            self.collided_speed_x = hitbox.collided_speed_x
            self.collided_speed_y = hitbox.collided_speed_y
            self.did_collide_x = hitbox.did_collide_x
            self.did_collide_y = hitbox.did_collide_y
            
    def did_collide_hitbox(self, obj):
        
        hitbox = self.get_hitbox()
        obj_hitbox = obj.get_hitbox()
        
        future_hitbox = hitbox.get_future_rect()
        future_obj_hitbox= obj_hitbox.get_future_rect()
        
        return future_hitbox.colliderect(future_obj_hitbox)
    
    def draw(self): 
        pass
    
    def cleanup(self):
        pass
        
        