import pygame, os, time
from pygame.locals import *
from gameObject import gameObject

class gameSprite(gameObject): 
    def __init__(self, gameWindow, left, top, speed, file_name):
        
        file_name = os.path.join("data", file_name)
        

        self.idle_sprite = pygame.sprite.Sprite()
        self.idle_sprite.image = pygame.image.load(file_name).convert_alpha()
        self.idle_sprite.mask = pygame.mask.from_surface(self.idle_sprite.image) 
        
        self.image = self.idle_sprite.image
        self.mask = self.idle_sprite.mask

        super(gameSprite, self).__init__(gameWindow, left, top, self.image.get_width(), self.image.get_height(), speed)
        
        self.type = "Sprite"
        self.movable = False
        
    def draw(self):
        
        my_rect = self.get_rect()
        screen_rect = self.gameWindow.get_rect()  
        
        if(my_rect.colliderect(screen_rect)):   
            self.gameWindow.screen.blit(self.image, (my_rect.left, my_rect.top))
            
    def set_sprite(self, sprite):
        
        self.image = sprite.image
        self.mask = sprite.mask
            
class animation:
    
    def __init__(self, frames, interval):
    
        self.num_frames = len(frames)
    
        if self.num_frames < 1:
            raise Exception("Cannot initialize an animation with zero frames!") 
    
        #Array of animation frames.  We take in file names, and load the images here in the init
        self.frames = []
        for frame in frames:
            file_name = os.path.join("data", frame)
            
            sprite = pygame.sprite.Sprite()
            sprite.image = pygame.image.load(file_name).convert_alpha()
            sprite.mask = pygame.mask.from_surface(sprite.image)
             
            self.frames.append(sprite)
        
        self.interval = interval
        
        self.current_frame = 0
        self.current_frame_time = 0
        
        
    def get_current_frame(self):
        
        if self.num_frames == 1:
            return self.frames[0]
        
        now = time.time()
        elapsed_time = now - self.current_frame_time
        
        if elapsed_time >= self.interval:
            
            self.current_frame += 1
            if(self.current_frame == self.num_frames):
                self.current_frame = 0
                
            self.current_frame_time = now
                
        return self.frames[self.current_frame]
        
        
        
    