import pygame, sys
from pygame.locals import *
from time import sleep 
from game import game
import time

class splashScreen(game):
    
    def __init__(self, screen, text_list, delay=2):
        
        self.text_list = text_list
        
        self.num_screens = len(text_list)
        self.current_screen = 0
        
        self.gameScreen = screen
        self.delay = delay
        self.time_started = 0
        
        self.line_len = 40
        
        self.state = "play"
        
    def show(self):
        self.time_started = time.time()
        self.game_loop()
        
    def update(self):
    
        
        for event in pygame.event.get(): 
            
            if event.type == QUIT: 
                sys.exit(0)
            
            elapsed_time = time.time() - self.time_started
            
            if self.current_screen == 0 and elapsed_time < self.delay: 
                pass
            else:
            
                if event.type == pygame.KEYUP:
                    self.current_screen += 1
                

                
    def draw(self):
        
        if(self.current_screen > (self.num_screens - 1)):
            self.state = "quit"
            return
        
        pygame.init()
        
        self.gameScreen.screen.fill([0,0,0])
        
        lines = self.parse_text(self.text_list[self.current_screen])
        
        font = pygame.font.Font(None, 20)
        rendered_lines = []
        total_height = 0
        longest_width = 0
        for line in lines:
            rendered_line = font.render(line, 1, (255, 255, 255))
            rendered_lines.append(rendered_line)
            
            text_rect = rendered_line.get_rect()
            total_height += text_rect.height
            if(text_rect.width > longest_width):
                longest_width = text_rect.width
        
        #Put text in center of screen
        text_left = (self.gameScreen.window_width / 2) - (longest_width / 2)
        text_top_offset = (self.gameScreen.window_height / 2) - (total_height / 2)
        
        for rendered_line in rendered_lines:
            text_rect = rendered_line.get_rect()
            final_text_rect = pygame.Rect(text_left, text_top_offset, text_rect.width, text_rect.height)
            self.gameScreen.screen.blit(rendered_line, final_text_rect)
            
            text_top_offset += text_rect.height
        
        pygame.display.flip() 
        
    def parse_text(self, text):
        
        lines = []
        current_line = ""
        current_word = ""
        
        for i in xrange(0, len(text)):
            
            if(text[i] != "\n"):
                current_word += text[i]
            
            #Parse out word tokens
            if(text[i] == "\n" or text[i] == " " or text[i]=="."):
                current_line += current_word
                current_word = ""
                    
            if len(current_line) >= self.line_len or text[i] == "\n":
                        
                lines.append(current_line)
                current_line = ""
                continue 
            
        current_line += current_word
            
        if len(current_line) > 0:
            lines.append(current_line)
            
        return lines
            
        
        